==> atmos-2.97.12.tar.gz: atmos-2.97.12/src/table_storage.h <==
/*
    $Id: table_storage.h 328 2024-10-30 12:37:15Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _TABLE_STORAGE_H
#define _TABLE_STORAGE_H

#include <algorithm>
#include <functional>
#include <vector>

#include <boost/iterator/transform_iterator.hpp>

#include "unstable_remove_if.h"

template< class T > class table_storage {
public:
	typedef T value_type;
	typedef std::vector<T> container_type;
	typedef typename container_type::size_type size_type;
	typedef typename container_type::const_reference const_reference;

	typedef typename container_type::iterator       iterator;
	typedef typename container_type::const_iterator const_iterator;

private:
	container_type m_storage;
public:
	table_storage() {
	}
	table_storage( size_type reserve ) {
		m_storage.reserve( reserve );
	}
	void swap( table_storage& ts ) throw() {
		m_storage.swap( ts.m_storage );
	}
	table_storage( const table_storage& ts ): m_storage( ts.m_storage ) {
	}
	table_storage& operator=( const table_storage& ts ) {
		 table_storage temp( ts );
		 temp.swap( *this );
		 return *this;
	}
	~table_storage() {
	}

	const_reference back()  const { return m_storage.back(); }
	const_reference front() const { return m_storage.front(); }
	void push_back( const T& x ) { m_storage.push_back( x ); }
	void pop_back() { m_storage.pop_back(); }

	iterator begin() { return m_storage.begin(); }
	iterator end()   { return m_storage.end(); }
	const_iterator begin() const { return m_storage.begin(); }
	const_iterator end()   const { return m_storage.end(); }

	size_type size() const { return m_storage.size(); }
	void clear() { return m_storage.clear(); }

	template<class Predicate> void unstable_remove_if( Predicate pred ) {
		using utils::algo::unstable_remove_if;
		m_storage.erase( unstable_remove_if( m_storage.begin(), m_storage.end(), pred ), m_storage.end() );
	}
	template<class Predicate> void remove_if( Predicate pred ) {
		using std::remove_if;
		m_storage.erase( remove_if( m_storage.begin(), m_storage.end(), pred ), m_storage.end() );
	}
};

namespace detail {
template<class Result, class Iterator> struct table_storage_column_traits {
	typedef Iterator subiterator;
	typedef typename std::iterator_traits< subiterator >::reference arg_type;
	typedef Result result_type;
	typedef std::function< result_type (arg_type) > function_type;
	typedef boost::transform_iterator< function_type, subiterator, result_type > iterator;
};
}

template<class Storage, class T> class table_storage_column_getter{
public:
	typedef Storage storage_type;
	typedef typename storage_type::const_iterator subiterator;
	typedef const T& result_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::function_type function_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::iterator      iterator;
private:
	const storage_type& m_storage;
	function_type m_function;
public:
	table_storage_column_getter( const storage_type& storage, const function_type& function ): m_storage(storage), m_function(function) {}

	iterator begin() const { return iterator( m_storage.begin(), m_function ); }
	iterator end() const   { return iterator( m_storage.end(), m_function ); }
};

template<class Storage, class T> class table_storage_column_setter{
public:
	typedef Storage storage_type;
	typedef typename storage_type::iterator subiterator;
	typedef T& result_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::function_type function_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::iterator      iterator;
private:
	storage_type& m_storage;
	function_type m_function;
public:
	table_storage_column_setter( storage_type& storage, const function_type& function ): m_storage(storage), m_function(function) {}

	iterator begin() { return iterator( m_storage.begin(), m_function ); }
	iterator end()   { return iterator( m_storage.end(), m_function ); }
};

template<class Storage, class T> class table_storage_column_generator{
public:
	typedef Storage storage_type;
	typedef typename storage_type::const_iterator subiterator;
	typedef T result_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::function_type function_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::iterator      iterator;
private:
	const storage_type& m_storage;
	function_type m_function;
public:
	table_storage_column_generator( const storage_type& storage, const function_type& function ): m_storage(storage), m_function(function) {}

	iterator begin() const { return iterator( m_storage.begin(), m_function ); }
	iterator end() const   { return iterator( m_storage.end(), m_function ); }
};

template<class Storage, class T> class table_storage_column_proxy{
public:
	typedef Storage storage_type;
	typedef typename storage_type::iterator       subiterator;
	typedef typename storage_type::const_iterator const_subiterator;
	typedef T& result_type;
	typedef const T& const_result_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::function_type function_type;
	typedef typename detail::table_storage_column_traits< result_type, subiterator >::iterator      iterator;
	typedef typename detail::table_storage_column_traits< const_result_type, const_subiterator >::function_type const_function_type;
	typedef typename detail::table_storage_column_traits< const_result_type, const_subiterator >::iterator      const_iterator;
private:
	storage_type& m_storage;
	function_type m_function;
	const_function_type m_const_function;
public:
	table_storage_column_proxy( storage_type& storage, const function_type& function, const const_function_type& const_function ):
		m_storage(storage), m_function(function), m_const_function(const_function) {}
	template<class F> table_storage_column_proxy( storage_type& storage, const F& function ):
		m_storage(storage), m_function(function), m_const_function(function) {}

	iterator begin() { return iterator( m_storage.begin(), m_function ); }
	iterator end()   { return iterator( m_storage.end(), m_function ); }
	const_iterator begin() const { return const_iterator( m_storage.begin(), m_const_function ); }
	const_iterator end()   const { return const_iterator( m_storage.end(), m_const_function ); }
};

template<class T> void swap( table_storage<T>& t, table_storage<T>& u ) {
	t.swap( u );
}

#endif // _TABLE_STORAGE_H

==> atmos-2.98.1.tar.gz: atmos-2.98.1/src/table_storage.h <==
/*
    $Id: table_storage.h 168 2010-11-04 13:20:12Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _TABLE_STORAGE_H
#define _TABLE_STORAGE_H

#include <algorithm>
#include <vector>

#include <boost/iterator/transform_iterator.hpp>

#include "unstable_remove_if.h"

template< class T > class table_storage {
public:
	typedef T value_type;
	typedef std::vector<T> container_type;
	typedef typename container_type::size_type size_type;
	typedef typename container_type::const_reference const_reference;
	typedef typename container_type::reference reference;

	typedef typename container_type::iterator       iterator;
	typedef typename container_type::const_iterator const_iterator;

private:
	container_type m_storage;
public:
	table_storage() {
	}
	table_storage( size_type reserve ) {
		m_storage.reserve( reserve );
	}
	void swap( table_storage& ts ) throw() {
		m_storage.swap( ts.m_storage );
	}
	table_storage( const table_storage& ts ): m_storage( ts.m_storage ) {
	}
	table_storage& operator=( const table_storage& ts ) {
		 table_storage temp( ts );
		 temp.swap( *this );
		 return *this;
	}
	~table_storage() {
	}

	const_reference back()  const { return m_storage.back(); }
	const_reference front() const { return m_storage.front(); }
	reference back()  { return m_storage.back(); }
	reference front() { return m_storage.front(); }
	void push_back( const T& x ) { m_storage.push_back( x ); }
	void pop_back() { m_storage.pop_back(); }

	iterator begin() { return m_storage.begin(); }
	iterator end()   { return m_storage.end(); }
	const_iterator begin() const { return m_storage.begin(); }
	const_iterator end()   const { return m_storage.end(); }

	size_type size() const { return m_storage.size(); }
	void clear() { return m_storage.clear(); }

	template<class Predicate> void unstable_remove_if( Predicate pred ) {
		using utils::algo::unstable_remove_if;
		m_storage.erase( unstable_remove_if( m_storage.begin(), m_storage.end(), pred ), m_storage.end() );
	}
	template<class Predicate> void remove_if( Predicate pred ) {
		using std::remove_if;
		m_storage.erase( remove_if( m_storage.begin(), m_storage.end(), pred ), m_storage.end() );
	}
};

template<class T, class F> class table_storage_column {
public:
	typedef T                subvalue_type;
	typedef table_storage<T> table_storage_type;
	typedef F                proxy_function_type;

	typedef typename table_storage_type::const_iterator const_subiterator;

	typedef boost::transform_iterator< proxy_function_type, const_subiterator > const_iterator;
private:
	const table_storage<T>& m_storage;
	proxy_function_type m_proxy_function;
public:
	table_storage_column( const table_storage<T>& storage, const F& proxy_function ):
		m_storage( storage ), m_proxy_function( proxy_function ) {
	}

	const_iterator begin() const { return const_iterator( m_storage.begin(), m_proxy_function ); }
	const_iterator end()   const { return const_iterator( m_storage.end(), m_proxy_function ); }
};

template<class T> void swap( table_storage<T>& t, table_storage<T>& u ) {
	t.swap( u );
}

#endif // _TABLE_STORAGE_H
